from .array import test_array
from .boolean import test_boolean
from .enum import test_enum
from .faker import test_faker
from .integer import test_integer
from .number import test_number
from .object import test_object
from .string import test_string
